@@ -1,8 +1,8 @@
 /*
  * radio plugin for Xfce4.
  *
- * Copyright (c) 2006/2007 Stefan Ott, All rights reserved.
- *  
+ * Copyright (c) 2006-2008 Stefan Ott, All rights reserved.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -51,7 +51,7 @@ update_signal_bar (radio_gui* data)
 {
 	if (!data->on || !data->show_signal) {
 		gtk_widget_hide (data->signal_bar);
-		data->timeout_id = 0;
+		data->signal_timeout_id = 0;
 		return FALSE;
 	} else {
 		gtk_widget_show (data->signal_bar);
@@ -64,9 +64,10 @@ update_signal_bar (radio_gui* data)
 						COLOR_SIGNAL_LOW, &color);
 		gtk_widget_modify_bg (data->signal_bar, GTK_STATE_PRELIGHT,
 								&color);
-		if (data->timeout_id == 0) {
-			data->timeout_id = g_timeout_add(500, (GtkFunction)
-					update_signal_bar, (gpointer) data);
+		if (data->signal_timeout_id == 0) {
+			data->signal_timeout_id = g_timeout_add(500,
+					(GtkFunction) update_signal_bar,
+							(gpointer) data);
 		}
 		return TRUE;
 	}
@@ -107,6 +108,58 @@ update_label (radio_gui* data)
 	free (label);
 }
 
+static gboolean update_radio(radio_gui *data) {
+	struct video_audio vid_aud;
+	long freq;
+
+	if (!data->on) {
+		data->fd = open(data->device, O_RDONLY);
+
+		// should I return FALSE here to stop timer?
+		if (data->fd == -1)
+			return TRUE;
+	}
+
+	if (ioctl(data->fd, VIDIOCGAUDIO, &vid_aud))
+		perror("VIDIOCGAUDIO");
+
+	if (vid_aud.flags & VIDEO_AUDIO_MUTE) {
+		close(data->fd);		
+
+		// Radio is off.  Did I think it was on?
+		if (data->on) {
+			// manually clean up; radio_stop would call ioctl	
+			if (data->show_signal)
+				gtk_widget_hide (data->signal_bar);
+			data->on = 0;
+		}
+	} else {
+		// Radio is on.  Did I think it was off?
+		if (!data->on) {
+			// manually clean up; radio_start would tune the radio
+			gtk_tooltips_enable (data->tooltips);			
+			
+			data->on = 1;
+		}
+
+		if (ioctl(data->fd, VIDIOCGFREQ, &freq)) {
+			perror("VIDIOCGFREQ");
+			return TRUE; // should maybe be FALSE?
+		}
+
+		// this plugin deals with frequencies in hundredths of Mhz
+		freq *= 100;
+		freq /= data->freqfact;
+
+		data->freq = freq;
+	}	
+
+	update_label(data);
+	update_signal_bar(data);	
+
+	return TRUE;
+}
+
 static gboolean
 radio_start (radio_gui* data)
 {
@@ -128,11 +181,15 @@ radio_start (radio_gui* data)
 		(tuner.flags & VIDEO_TUNER_LOW))
 		data->freqfact = 16000;
 
-	if (ioctl (data->fd, VIDIOCSAUDIO, &vid_aud)) perror("VIDIOCGAUDIO");
+	if (ioctl (data->fd, VIDIOCGAUDIO, &vid_aud)) perror("VIDIOCGAUDIO");
 	if (vid_aud.volume == 0) vid_aud.volume = 65535;
 	vid_aud.flags &= ~VIDEO_AUDIO_MUTE;
 	if (ioctl (data->fd, VIDIOCSAUDIO, &vid_aud)) perror("VIDIOCSAUDIO");
 
+	if (strcmp(data->startup_command, "") != 0) {
+		xfce_exec(data->startup_command, FALSE, FALSE, NULL);
+	}
+
 	radio_tune (data);
 	gtk_tooltips_enable (data->tooltips);
 	return TRUE;
@@ -152,8 +209,8 @@ radio_stop (radio_gui* data)
 
 	if (data->show_signal) gtk_widget_hide (data->signal_bar);
 
-	if (strcmp (data->command, "") != 0) {
-		xfce_exec (data->command, FALSE, FALSE, NULL);
+	if (strcmp (data->shutdown_command, "") != 0) {
+		xfce_exec (data->shutdown_command, FALSE, FALSE, NULL);
 	}
 }
 
@@ -630,24 +687,56 @@ plugin_free (radio_gui *data)
 	if (data->on) radio_stop(data);
 	free_presets (data);
 	g_free (data);
+
+	/* TODO: check this
++	
++	// lex: I think this is necessary.
++	if (data->signal_timeout_id)
++		g_source_remove(data->signal_timeout_id);	
++	if (data->radio_timeout_id)
++		g_source_remove(data->radio_timeout_id);
++	
+ 	*/
 }
 
 static radio_gui *
 plugin_control_new (XfcePanelPlugin *plugin)
 {
+	int fd;
+	struct video_tuner tuner;
 	radio_gui* plugin_data = create_gui();
 
 	plugin_data->plugin = plugin;
 
 	plugin_data->on = FALSE;
 	plugin_data->freq = FREQ_INIT;
-	plugin_data->freqfact = 16;
-	plugin_data->show_signal = TRUE;
 	strcpy(plugin_data->device, "/dev/radio0");
+
+	// have to get the freqfact here now because someone else might
+	// turn on the radio
+	fd = open(plugin_data->device, O_RDONLY);
+	if (fd != -1) {
+		if (ioctl(fd, VIDIOCGTUNER, &tuner) == 0) {
+			if (tuner.flags & VIDEO_TUNER_LOW)
+				plugin_data->freqfact = 16000;
+			else
+				plugin_data->freqfact = 16;
+		} else {
+			plugin_data->freqfact = 16;
+		}
+		close (fd);
+	} else {
+		plugin_data->freqfact = 16;
+	}
+	plugin_data->show_signal = TRUE;
 	plugin_data->presets = NULL;
 	plugin_data->scroll = CHANGE_FREQ;
-	plugin_data->timeout_id = 0;
+	plugin_data->signal_timeout_id = 0;
+	plugin_data->radio_timeout_id = g_timeout_add(2000,
+			(GtkFunction)update_radio, (gpointer)plugin_data);
 	plugin_data->tooltips = gtk_tooltips_new();
+	plugin_data->startup_command[0] = '\0';
+	plugin_data->shutdown_command[0] = '\0';
 
 	update_label(plugin_data);
 
@@ -657,11 +746,19 @@ plugin_control_new (XfcePanelPlugin *plugin)
 }
 
 static void
-radio_command_changed (GtkEditable* editable, void *pointer)
+radio_startup_command_changed (GtkEditable* editable, void *pointer)
 {
 	radio_gui* data = (radio_gui*) pointer;
 	const char* command = gtk_entry_get_text (GTK_ENTRY (editable));
-	strncpy (data->command, command, MAX_COMMAND_LENGTH);
+	strncpy (data->startup_command, command, MAX_COMMAND_LENGTH);
+}
+
+static void
+radio_shutdown_command_changed (GtkEditable* editable, void *pointer)
+{
+	radio_gui* data = (radio_gui*) pointer;
+	const char* command = gtk_entry_get_text (GTK_ENTRY (editable));
+	strncpy (data->shutdown_command, command, MAX_COMMAND_LENGTH);
 }
 
 static void
@@ -712,7 +809,8 @@ radio_plugin_create_options (XfcePanelPlugin *plugin, radio_gui *data)
 	GSList *show_signal_group = NULL;	// signal strength:
 	GtkWidget *signal_show;			//  - show
 	GtkWidget *signal_hide;			//  - hide
-	GtkWidget *command_entry;		// post-down command
+	GtkWidget *startup_command_entry;	// post-down command
+	GtkWidget *shutdown_command_entry;	// post-down command
 	GtkWidget *device_entry;		// v4l-device
 	GSList *scroll_group = NULL;		// scroll action:
 	GtkWidget *frequency_button;		//  - change frequency
@@ -774,16 +872,32 @@ radio_plugin_create_options (XfcePanelPlugin *plugin, radio_gui *data)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (signal_hide),
 							!data->show_signal);
 
-	label = gtk_label_new (_("Execute command after shutdown"));
+	label = gtk_label_new (_("Execute command after startup"));
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL,
 								0, 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 
-	command_entry = gtk_entry_new_with_max_length (MAX_COMMAND_LENGTH);
-	gtk_entry_set_text (GTK_ENTRY (command_entry), data->command);
-	gtk_widget_show (command_entry);
-	gtk_table_attach (GTK_TABLE (table), command_entry, 1, 2, 3, 4,
+	startup_command_entry = gtk_entry_new_with_max_length
+							(MAX_COMMAND_LENGTH);
+	gtk_entry_set_text (GTK_ENTRY (startup_command_entry),
+							data->startup_command);
+	gtk_widget_show (startup_command_entry);
+	gtk_table_attach (GTK_TABLE (table), startup_command_entry, 1, 2, 3, 4,
+					GTK_EXPAND | GTK_FILL, 0, 0, 0);
+
+	label = gtk_label_new (_("Execute command after shutdown"));
+	gtk_widget_show (label);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL,
+								0, 0, 0);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+	shutdown_command_entry = gtk_entry_new_with_max_length
+							(MAX_COMMAND_LENGTH);
+	gtk_entry_set_text (GTK_ENTRY (shutdown_command_entry),
+							data->shutdown_command);
+	gtk_widget_show (shutdown_command_entry);
+	gtk_table_attach (GTK_TABLE (table), shutdown_command_entry, 1, 2, 4, 5,
 					GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
 	label = gtk_label_new (_("Mouse scrolling changes"));
@@ -825,14 +939,16 @@ radio_plugin_create_options (XfcePanelPlugin *plugin, radio_gui *data)
 	gtk_table_attach (GTK_TABLE(table), device_entry, 1, 2, 0, 1,
 					GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
-	g_signal_connect ((gpointer) command_entry, "changed",
-				G_CALLBACK (radio_command_changed), data);
+	g_signal_connect ((gpointer) startup_command_entry, "changed",
+			G_CALLBACK (radio_startup_command_changed), data);
+	g_signal_connect ((gpointer) shutdown_command_entry, "changed",
+			G_CALLBACK (radio_shutdown_command_changed), data);
 	g_signal_connect ((gpointer) device_entry, "changed",
-				G_CALLBACK (radio_device_changed), data);
+			G_CALLBACK (radio_device_changed), data);
 	g_signal_connect (G_OBJECT (signal_show), "toggled",
-				G_CALLBACK(radio_show_signal_changed), data);
+			G_CALLBACK(radio_show_signal_changed), data);
 	g_signal_connect (G_OBJECT (frequency_button), "toggled",
-				G_CALLBACK (radio_scroll_type_changed), data);
+			G_CALLBACK (radio_scroll_type_changed), data);
 
 	gtk_widget_show (dialog);
 }
@@ -851,7 +967,10 @@ write_config (XfcePanelPlugin *plugin, radio_gui *data)
 		xfce_rc_set_group	(rc, "radio plugin");
 
 		xfce_rc_write_entry	(rc, "dev", data->device);
-		xfce_rc_write_entry	(rc, "cmd", data->command);
+		xfce_rc_write_entry	(rc, "startup_cmd",
+							data->startup_command);
+		xfce_rc_write_entry	(rc, "shutdown_cmd",
+							data->shutdown_command);
 		xfce_rc_write_int_entry (rc, "frq", data->freq);
 		xfce_rc_write_int_entry (rc, "scroll", data->scroll);
 		xfce_rc_write_bool_entry(rc, "show_signal", data->show_signal);
@@ -897,8 +1016,11 @@ read_config (XfcePanelPlugin *plugin, radio_gui *data)
 	if ((value = xfce_rc_read_entry (rc, "dev", NULL)) && *value) {
 		strncpy (data->device, value, MAX_DEVICE_NAME_LENGTH);
 	}
-	if ((value = xfce_rc_read_entry (rc, "cmd", NULL)) && *value) {
-		strncpy (data->command, value, MAX_COMMAND_LENGTH);
+	if ((value = xfce_rc_read_entry (rc, "startup_cmd", NULL)) && *value) {
+		strncpy (data->startup_command, value, MAX_COMMAND_LENGTH);
+	}
+	if ((value = xfce_rc_read_entry (rc, "shutdown_cmd", NULL)) && *value) {
+		strncpy (data->shutdown_command, value, MAX_COMMAND_LENGTH);
 	}
 
 	free_presets (data);
