@@ -1,7 +1,7 @@
 /*
  * radio plugin for Xfce4.
  *
- * Copyright 2006/2007 Stefan Ott, All rights reserved.
+ * Copyright 2006-2008 Stefan Ott, All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 
 #define MAX_LABEL_LENGTH	7
 #define MAX_DEVICE_NAME_LENGTH	32
-#define MAX_COMMAND_LENGTH	128
+#define MAX_COMMAND_LENGTH	512
 #define MAX_PRESET_NAME_LENGTH	15
 
 #define COLOR_SIGNAL_LOW	"#ffff00"
@@ -61,7 +61,8 @@ typedef struct {
 	GtkWidget*		signal_bar;
 	GtkTooltips*		tooltips;
 	
-	int			timeout_id;
+	int			signal_timeout_id;
+	int			radio_timeout_id;
 
 	gboolean		on;
 	gboolean		show_signal;
@@ -69,7 +70,8 @@ typedef struct {
 	int			fd;
 	int			freqfact;
 	char			device [MAX_DEVICE_NAME_LENGTH];
-	char			command [MAX_COMMAND_LENGTH];
+	char			startup_command [MAX_COMMAND_LENGTH];
+	char			shutdown_command [MAX_COMMAND_LENGTH];
 	radio_preset*		presets;
 	mouse_scroll_reaction	scroll;
 
